/*
 * Copyright (c) 2019-2020, NVIDIA CORPORATION. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#pragma once

#include "GfxPlatform.h"
#include "SDLWrapper.h"

using namespace vr;

// putting all the old datatypes in here as clientopenvr is still using them as interposers
// to the new input system.

typedef enum
{
    cxrButton_System,
    cxrButton_ApplicationMenu,

    cxrButton_Grip_Touch,
    cxrButton_Grip_Click,

    cxrButton_Trigger_Touch,
    cxrButton_Trigger_Click,

    cxrButton_Touchpad_Touch,
    cxrButton_Touchpad_Click,

    cxrButton_Joystick_Touch,
    cxrButton_Joystick_Click,

    cxrButton_A,
    cxrButton_B,

    cxrButton_X,
    cxrButton_Y,

    cxrButton_Num
} cxrButtonId;

typedef enum
{
    cxrAnalog_Trigger,

    cxrAnalog_TouchpadX,
    cxrAnalog_TouchpadY,

    cxrAnalog_JoystickX,
    cxrAnalog_JoystickY,

    cxrAnalog_Grip,
    cxrAnalog_Grip_Force,

    cxrAnalog_Num
} cxrAnalogId;

typedef enum
{
    cxrControllerType_HtcVive,
    cxrControllerType_ValveIndex,
    cxrControllerType_OculusTouch,
    cxrControllerType_None
} cxrControllerType;

class VisionController
{
public:
    void Init(ETrackedControllerRole role, cxrControllerType type);
    void FireActions(cxrReceiverHandle m_receiver, cxrControllerHandle handle, bool remapSysBtn);
    void TriggerHaptic(const cxrHapticFeedback *haptic);

    VRActionHandle_t            m_booleanInputs[cxrButtonId::cxrButton_Num] = {};
    VRActionHandle_t            m_analogInputs[cxrAnalogId::cxrAnalog_Num] = {};
    VRActionHandle_t            m_haptic = 0;
    ETrackedControllerRole      m_role = ETrackedControllerRole::TrackedControllerRole_Invalid;
    cxrControllerType           m_type = cxrControllerType_None;
};

class VisionClientOpenVR : public ClientCommon
{
public:
    VisionClientOpenVR(bool noBtnRemap);
    ~VisionClientOpenVR();

    // ClientCommon
    cxrError Init(const CloudXR::ClientOptions& options, const std::string &outputPath);

    // cxrClientCallbacks
    void GetTrackingState(cxrVRTrackingState *trackingState);
    void TriggerHaptic(const cxrHapticFeedback *haptic);
    cxrBool RenderAudio(const cxrAudioFrame *audioFrame);

protected:
    // ClientCommon
    void GetChaperone(cxrChaperone *chaperone);
    void GetGraphicsContext(cxrGraphicsContext *context, cxrGraphicsContextType type);
    cxrError SubmitFrame(cxrFramesLatched* framesLatched) override;
    bool Update();

    SDLWrapper              m_sdl;
    GfxPlatform             m_platform;
    vr::VRActionSetHandle_t m_actionset = vr::k_ulInvalidActionSetHandle;
    VisionController        m_controllers[2];
    cxrVRTrackingState      m_trackingState;
    IVRSystem*              m_hmd;
    ETrackingUniverseOrigin m_universeOrigin = ETrackingUniverseOrigin::TrackingUniverseRawAndUncalibrated;
    bool                    m_init;
    bool                    m_remapSysBtn;
    cxrControllerHandle     m_newControllers[2] = {};
};
