/*
 * Copyright (c) 2019-2020, NVIDIA CORPORATION. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#pragma once

#include "SDLWrapper.h"
#include "GfxPlatform.h"
#include <stdint.h>

class VisionClientWindow : public ClientCommon
{
public:
    VisionClientWindow(cxrGraphicsContextType type);
    ~VisionClientWindow();

    // ClientCommon
    cxrError Init(const CloudXR::ClientOptions& options, const std::string &outputPath);

    // cxrClientCallbacks
    void GetTrackingState(cxrVRTrackingState *trackingState);
    void TriggerHaptic(const cxrHapticFeedback *haptic) { };
    cxrBool RenderAudio(const cxrAudioFrame *audioFrame);

protected:
    // ClientCommon
    void GetChaperone(cxrChaperone *chaperone);
    void GetGraphicsContext(cxrGraphicsContext *context, cxrGraphicsContextType type);
    cxrError SubmitFrame(cxrFramesLatched* framesLatched) override;
    bool Update();

    uint32_t     m_width;
    uint32_t     m_height;
    SDLWrapper   m_sdl;
    GfxPlatform  m_platform;
    int          m_frameCounter;
    bool         m_buttonDown;
    float        m_triggerVal = 0.0f;
    cxrControllerHandle     m_newControllers[2] = {};
};
