/*
 * Copyright (c) 2019-2020, NVIDIA CORPORATION. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#pragma once

#ifndef _WIN32
#include <X11/Xlib.h>
#endif

struct SDL_Window;
typedef void *SDL_GLContext;
typedef uint32_t SDL_AudioDeviceID;
typedef void(*AudioRecordingCallback) (void *userdata, uint8_t * stream, int len);

class SDLWrapper
{
    SDL_Window* m_window = nullptr;
    SDL_GLContext m_glContext = 0;
    SDL_AudioDeviceID m_playbackDevice = 0;
    SDL_AudioDeviceID m_recordingDevice = 0;
    unsigned m_bytesPerMs = 0;
    bool m_shouldQuit = false;

public:
    ~SDLWrapper();

    bool InitVideo();
    bool InitWindow(const char* title, int width, int height, bool isOpenGL);
    void InitAudio(int frequency, uint8_t channels, unsigned bufferMs, bool& playback, bool& record,
        AudioRecordingCallback audioRecordingCallback, void* audioRecordingContext);
    void FillMonitorParams(uint32_t &width, uint32_t &height, uint32_t &hz);
    void Resize(int width, int height);

#ifdef _WIN32
    HDC GetHDC();
    HWND GetHWND();
#else
    Display *GetDpy();
    Window GetWin();
#endif

    unsigned GetMouseState(int *x, int *y);
    void PollEvents();
    bool ShouldQuit() const { return m_shouldQuit; }

    void QueueAudio(const void *data, uint32_t len);
};
