/*
 * Copyright (c) 2021-2023, NVIDIA CORPORATION. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

import UIKit

class ViewController: UIViewController {

    @IBOutlet weak var addrField: UITextField!
    @IBOutlet weak var btnConnectXR: UIButton!

    override func viewDidLoad() {
        super.viewDidLoad()
        if let address = UserDefaults.standard.string(forKey: "ServerAddress") {
            addrField.text = address
        } else {
            btnConnectXR.isEnabled = false
        }
    }
    
    @IBAction func addrChanged(_ sender: Any) {
        if addrField.text?.count ?? 0 > 0 {
            btnConnectXR.isEnabled = true
        }
        else {
            btnConnectXR.isEnabled = false
        }
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if let cxrView = segue.destination as? CXRViewController {
            cxrView.address = addrField.text!
            UserDefaults.standard.set(addrField.text!, forKey: "ServerAddress")
        }
    }
}

